#include <tango.h>
#include <string>
#include <thread>  
#include <chrono>
#include <cstdlib>
#include <unistd.h>
#include <csignal>

#include <sys/time.h>
#include <ctime>

#include "timedebug.hh"

struct sigaction old_action;

Tango::DeviceProxy * pproxy = 0;


void stopProxy(){
  if(pproxy)
    pproxy->command_inout("StopMove");
    delete pproxy;

}

void sigint_handler(int sig_no)
{
  std::cout << "CTRL-C pressed\n" << std::cout;
  stopProxy();
  sigaction(SIGINT, &old_action, NULL);
  kill(0, SIGINT);
}

/*
void get_wall_time(struct timeval& time){
  gettimeofday(&time,NULL);
  return; 
}


const std::string currentDateTime() {
  std::time_t now = std::time(0);
  struct tm  tstruct;
  char       buf[80];
  tstruct = *std::localtime(&now);
  std::strftime(buf, sizeof(buf), "%Y-%m-%d.%X", &tstruct);

  return buf;
}

*/
void move(Tango::DeviceProxy & proxy, double position){
  Tango::DevState state;
  //  double wall0, wall1;
  struct timeval wtime0;
  struct timeval wtime1;
  double rpos;
  Tango::DeviceAttribute pdata("Position", position);
  get_wall_time(wtime0);
  try{ 
    proxy.write_attribute(pdata);
  }
  catch(Tango::DevFailed &e){
    Tango::Except::print_exception(e);
  }
  catch(std::string &e){
    std::cerr << "ERROR: " << e << std::endl;
  }
  Tango::DeviceData stat = proxy.command_inout("State");
  stat >> state;
  //  std::cout  << "state" << state << Tango::MOVING << std::endl;
  while(state == Tango::MOVING){
    try{ 
      std::this_thread::sleep_for (std::chrono::seconds(1));
      Tango::DeviceData stat = proxy.command_inout("State");
      stat >> state;
      get_wall_time(wtime0);
      Tango::DeviceAttribute patt;
      patt = proxy.read_attribute("Position");
      patt >> rpos;
    }
    catch(Tango::DevFailed &e){
      Tango::Except::print_exception(e);
    }
    catch(std::string &e){
      std::cerr << "ERROR: " << e << std::endl;
    }
    get_wall_time(wtime1);

    std::cout 
      << "POS " << rpos 
      << " DIFF " 
      << (double)(wtime1.tv_sec - wtime0.tv_sec)+ (double)(wtime1.tv_usec - wtime0.tv_usec)*0.000001 << " " 
      << "START "
      << wtime0.tv_sec << " " << wtime0.tv_usec << " " 
      << "STOP "
      << wtime1.tv_sec << " " << wtime1.tv_usec << " " 
      << currentDateTime() << std::endl;
    
  }
  
}

int main(int argc, char ** argv){

  struct sigaction action;
  memset(&action, 0, sizeof(action));
  action.sa_handler = &sigint_handler;
  sigaction(SIGINT, &action, &old_action);



  // timeout for servers in msec  
  int timeout = 24000;
  // device name
  std::string serverName;
  // device proxy poineter
  
  double position;

  if(argc < 3){
    std::cout << "testmotor <device> <position>" << std::endl;
    return -1;
  }

  serverName = std::string(argv[1]);
  position = std::atof(argv[2]);
  pproxy = new Tango::DeviceProxy(serverName);
  pproxy->set_timeout_millis(timeout);

  move(*pproxy, position);

  stopProxy();

}
