#include <tango.h>
#include <string>
#include <thread>  
#include <chrono>

#include <sys/time.h>
#include <ctime>

#include "timedebug.hh"

void check(Tango::DeviceProxy & proxy){
  Tango::DevState state;
  struct timeval wtime0;
  struct timeval wtime1;
  while(true){
    try{ 
      std::this_thread::sleep_for (std::chrono::seconds(1));
      get_wall_time(wtime0);
      state = proxy.state();
    }
    catch(Tango::DevFailed &e){
      Tango::Except::print_exception(e);
    }
    catch(std::string &e){
      std::cerr << "ERROR: " << e << std::endl;
    }
    get_wall_time(wtime1);

    std::cout 
      << " DIFF " 
      << (double)(wtime1.tv_sec - wtime0.tv_sec)
      + (double)(wtime1.tv_usec - wtime0.tv_usec)*0.000001 << " " 
      << "START "
      << wtime0.tv_sec << " " << wtime0.tv_usec << " " 
      << "STOP "
      << wtime1.tv_sec << " " << wtime1.tv_usec << " " 
      << currentDateTime() << std::endl;
    
  }  
}

int main(int argc, char ** argv){

  // timeout for servers in msec  
  int timeout = 24000;
  // device name
  std::string serverName;
  // device proxy poineter
  Tango::DeviceProxy * pproxy = 0;

  if(argc < 2){
    std::cout << "servertest <device>" << std::endl;
    return -1;
  }

  serverName = std::string(argv[1]);
  pproxy = new Tango::DeviceProxy(serverName);
  pproxy->set_timeout_millis(timeout);

  check(*pproxy);

  if(pproxy)
    delete pproxy;

}
